import field    # Die Verwaltung des Spielbretts, Eingabe von Zuegen, Gewinnkondition
import ui       # Interaktion mit dem Anwender, Ausgabe des Brettes und Eingabe von Zuegen.
import ai       # Der Computerspieler.

# Ablauf des Spiels, Eingabe von Optionen.

# Hauptprogramm:
# Liefert das "Startmenue" des Spieles, und startet die einzelnen Partien.
def main_menu():
    running = 1
    print "Willkommen bei Vier Gewinnt!"

    # Diese Schleife laeuft bis zum Abbruch durch den Anwender.
    while running:
        ai_opt, width, height, length = 2, 7, 6, 4
        print "\n\n"
        print "+=====================================================+"
        print "|                     Hauptmenue                      |"
        print "+-----------------------------------------------------+"
        print "| Optionen:                                           |"
        print "|                                                     |"
        print "| start  - neues Spiel, Mensch beginnt, 6x7, 4 Steine |"
        print "| custom - andere Optionen auswaehlen                 |"
        print "| quit   - beendet das Programm                       |"
        print "+=====================================================+\n\n"

        choice = ui.prompt_choice(["start", "custom", "quit"])
        if choice == "custom":
            ai_opt, width, height, length = get_user_options()
        elif choice == "quit":
            break

        # Wenn der Anwender die Eingabe abgebrochen hat, entweder beenden oder
        # von neuem anfangen.
        if ai_opt == 'quit':
            break
        elif ai_opt == 'reset':
            continue

        # Starte neue Runden, bis der Spieler abbricht."
        more_games = 1
        while more_games:
            state = run_game(ai_opt, width, height, length)
            # Wenn das Spiel abgebrochen wurde, entweder zurueck ins Hauptmenue
            # oder ein neues Spiel beginnen.
            if state == 'quit':
                running = 0
                break
            elif state == 'menu':
                break
            elif state == 'restart':
                continue
            else:
                print "Noch eine Runde, oder ins Hauptmenue zurueck?"
                choice = ui.prompt_choice(['quit', 'menu', 'restart'])
                if choice == 'quit':
                    running = 0
                    more_games = 0
                elif choice == 'menu':
                    more_games = 0

    print "Auf Wiedersehen!"



# Durchlaeuft ein Spiel.
# Der Rueckgabewert kann die folgenden Werte einnehmen:
# r - Spiel abgebrochen, Neustart.
# q - Spiel abgebrochen, zum Hauptmenue zurueck
# 0 - Unentschieden
# 1 - Spieler A [X] gewinnt.
# 2 - Spieler B [O] gewinnt.
def run_game(ai_opt, width, height, length):
    players = ["-", "X", "O"]
    names = ["", "Mensch", "Computer", ""]
    if ai_opt == 1:
        names.reverse()
        
    print "\n\n\nEin neues Spiel hat begonnen. \n\n\
     Das Feld ist {0}x{1} gross. Der Sieger ist, wer zuerst eine horizontale, \n\
     vertikale oder diagonale Reihe mit {2} Steinen besitzt."\
     .format(width, height, length)
    print
    print "Spieler {0} = {1}".format(players[1], names[1])
    print "Spieler {0} = {1}".format(players[2], names[2])
    print
    
    board = field.init_field(width, height)
    player = 1
    column = -1
    
    while player:
        quit_opts = ['menu', 'quit', 'restart']
        ui.print_field(board)
        moves = field.legal_moves(board)
        if len(moves) == 0:
            print "Es sind keine weiteren Zuege moeglich. Das Spiel endet unentschieden."
            return 0
            
        if (player == ai_opt):
            print "Computerspieler {0} ist am Zug.".format(players[player])
            # Der Computer untersucht das Brett auf /offensichtlich/ gute Zuege.
            column = ai.prompt_move(board, ai_opt, length)
        else:
            print "Spieler {0}, Sie sind am Zug.".format(players[player])
            # Drucke gegebenenfalls den letzten Zug aus.
            if column >= 0:
                ui.print_move(column)
            print "Bitte geben Sie Ihren naechsten Spielzug ein."
            # Der Mensch sieht das Brett und braucht nur die gueltigen Zuege.
            choice = ui.prompt_choice(moves + quit_opts)


            # Wenn der Spieler abbrechen will, gib den Abbruchzustand zurueck.
            if choice in quit_opts:
                print "Spiel wurde vorzeitig abgebrochen."
                return choice

            # prompt_choice hat die Eingabe schon validiert.
            column = int(choice)

        row = field.insert_move(board, column, player)
        won = field.is_won(board, length, player)
        if won:
            # Drucke das board zum Abschluss noch einmal aus.
            ui.print_field(board)
            print "Spieler {0} ({1}) hat das Spiel gewonnen!".format(players[player], names[player])
            return player

        player = 2 if player == 1 else 1



def get_user_options():
    # Verwende dieselbe Input-Methode, die auch zur Erfragung
    # von Spielzuegen benutzt wird. Das heisst auch, dass der
    # Benutzer jederzeit "restart" und "quit" waehlen kann.
    quit_opts = ['quit', 'reset']
    print "Wollen Sie gegen den Computer antreten?"
    print "  1) Kein Computergegner"
    print "  2) Der Computer beginnt"
    print "  3) Sie beginnen"
    
    choice = ui.prompt_choice([1, 2, 3] + quit_opts)
    if choice not in quit_opts:
        ai_opt = int(choice) - 1
    else:
        return choice, 0, 0, 0

    print "Geben Sie bitte die Breite des Spielfeldes ein (Standard: 7)"
    choice = ui.prompt_choice(range(4, 20) + quit_opts)
    if choice not in quit_opts:
        width = int(choice)
    else:
        return choice, 0, 0, 0

    print "Geben Sie bitte die Hoehe des Spielfeldes ein (Standard: 6)"
    choice = ui.prompt_choice(range(4, 20) + quit_opts)
    if choice not in quit_opts:
        height = int(choice)
    else:
        return choice, 0, 0, 0

    print "Nach wie vielen Steinen hat ein Spieler gewonnen? (Standard: 4)"
    choice = ui.prompt_choice(range(3, 8) + quit_opts)
    if choice not in quit_opts:
        length = int(choice)
    else:
        return choice, 0, 0, 0

    return ai_opt, width, height, length    
